# Generated by Django 2.2.1 on 2019-05-22 19:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AnsibleRemote',
            fields=[
                ('remote_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Remote')),
            ],
            options={
                'abstract': False,
            },
            bases=('core.remote',),
        ),
        migrations.CreateModel(
            name='CollectionRemote',
            fields=[
                ('remote_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Remote')),
                ('whitelist', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=('core.remote',),
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('content_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Content')),
                ('namespace', models.CharField(max_length=64)),
                ('name', models.CharField(max_length=64)),
                ('version', models.CharField(max_length=128)),
            ],
            options={
                'unique_together': {('version', 'name', 'namespace')},
            },
            bases=('core.content',),
        ),
        migrations.CreateModel(
            name='Collection',
            fields=[
                ('content_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Content')),
                ('namespace', models.CharField(max_length=64)),
                ('name', models.CharField(max_length=64)),
                ('version', models.CharField(max_length=128)),
            ],
            options={
                'unique_together': {('namespace', 'name', 'version')},
            },
            bases=('core.content',),
        ),
        migrations.CreateModel(
            name='AnsibleDistribution',
            fields=[
                ('basedistribution_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.BaseDistribution')),
                ('repository', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Repository')),
                ('repository_version', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.RepositoryVersion')),
            ],
            options={
                'abstract': False,
            },
            bases=('core.basedistribution',),
        ),
    ]
